@@ -77,6 +77,7 @@ extern "C" {
 #include "asterisk/res_srtp.h"
 #include "asterisk/stasis.h"
 #include "asterisk/vector.h"
+#include "asterisk/logger_category.h"
 
 /*! Maximum number of payload types RTP can support. */
 #define AST_RTP_MAX_PT 128
@@ -589,6 +590,26 @@ struct ast_rtp_engine_dtls {
 	const char *(*get_fingerprint)(struct ast_rtp_instance *instance);
 };
 
+#ifdef TEST_FRAMEWORK
+/*! \brief Structure that represents the test functionality for res_rtp_asterisk unit tests */
+struct ast_rtp_engine_test {
+	/*! Drops RTP packets while this has a value greater than 0 */
+	int packets_to_drop;
+	/*! Sends a SR/RR instead of RTP the next time RTP would be sent */
+	int send_report;
+	/*! Set to 1 whenever SDES is received */
+	int sdes_received;
+	/*! Get the number of packets in the receive buffer for a RTP instance */
+	size_t (*recv_buffer_count)(struct ast_rtp_instance *instance);
+	/*! Get the maximum number of packets the receive buffer can hold for a RTP instance */
+	size_t (*recv_buffer_max)(struct ast_rtp_instance *instance);
+	/*! Get the number of packets in the send buffer for a RTP instance */
+	size_t (*send_buffer_count)(struct ast_rtp_instance *instance);
+	/*! Set the schedid for RTCP */
+	void (*set_schedid)(struct ast_rtp_instance *instance, int id);
+};
+#endif
+
 /*! Structure that represents an RTP stack (engine) */
 struct ast_rtp_engine {
 	/*! Name of the RTP engine, used when explicitly requested */
@@ -670,6 +691,10 @@ struct ast_rtp_engine {
 	struct ast_rtp_engine_ice *ice;
 	/*! Callback to pointer for optional DTLS SRTP support */
 	struct ast_rtp_engine_dtls *dtls;
+#ifdef TEST_FRAMEWORK
+	/*! Callback to pointer for test callbacks for RTP/RTCP unit tests */
+	struct ast_rtp_engine_test *test;
+#endif
 	/*! Callback to enable an RTP extension (returns non-zero if supported) */
 	int (*extension_enable)(struct ast_rtp_instance *instance, enum ast_rtp_extension extension);
 	/*! Linked list information */
@@ -2513,6 +2538,18 @@ int ast_rtp_engine_unload_format(struct ast_format *format);
  */
 struct ast_rtp_engine_ice *ast_rtp_instance_get_ice(struct ast_rtp_instance *instance);
 
+#ifdef TEST_FRAMEWORK
+/*!
+ * \brief Obtain a pointer to the test callbacks on an RTP instance
+ *
+ * \param instance the RTP instance
+ *
+ * \retval test callbacks if present
+ * \retval NULL if not present
+ */
+struct ast_rtp_engine_test *ast_rtp_instance_get_test(struct ast_rtp_instance *instance);
+#endif
+
 /*!
  * \brief Obtain a pointer to the DTLS support present on an RTP instance
  *
@@ -2686,6 +2723,70 @@ struct stasis_message_type *ast_rtp_rtcp_sent_type(void);
  */
 struct stasis_message_type *ast_rtp_rtcp_received_type(void);
 
+#ifdef TEST_FRAMEWORK
+/*!
+ * \brief Get the maximum size of the receive buffer
+ *
+ * \param instance The RTP instance
+ * \retval The recv_buffer max size if it exists, else 0
+ */
+size_t ast_rtp_instance_get_recv_buffer_max(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Get the current size of the receive buffer
+ *
+ * \param instance The RTP instance
+ * \retval The recv_buffer size if it exists, else 0
+ */
+size_t ast_rtp_instance_get_recv_buffer_count(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Get the current size of the send buffer
+ *
+ * \param instance The RTP instance
+ * \retval The send_buffer size if it exists, else 0
+ */
+size_t ast_rtp_instance_get_send_buffer_count(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Set the schedid for RTCP
+ *
+ * \param instance The RTP instance
+ * \param id The number to set schedid to
+ */
+void ast_rtp_instance_set_schedid(struct ast_rtp_instance *instance, int id);
+
+/*!
+ * \brief Set the number of packets to drop on RTP read
+ *
+ * \param instance The RTP instance
+ * \param num The number of packets to drop
+ */
+void ast_rtp_instance_drop_packets(struct ast_rtp_instance *instance, int num);
+
+/*!
+ * \brief Sends a SR/RR report the next time RTP would be sent
+ *
+ * \param instance The RTP instance
+ */
+void ast_rtp_instance_queue_report(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Get the value of sdes_received on the test engine
+ *
+ * \param instance The RTP instance
+ * \retval 1 if sdes_received, else 0
+ */
+int ast_rtp_instance_get_sdes_received(struct ast_rtp_instance *instance);
+
+/*!
+ * \brief Resets all the fields to default values for the test engine
+ *
+ * \param instance The RTP instance
+ */
+void ast_rtp_instance_reset_test_engine(struct ast_rtp_instance *instance);
+#endif
+
 /*!
  * \brief Convert given stat instance into json format
  * \param stats
@@ -2700,6 +2801,17 @@ struct ast_json *ast_rtp_convert_stats_json(const struct ast_rtp_instance_stats
  */
 struct ast_json *ast_rtp_instance_get_stats_all_json(struct ast_rtp_instance *instance);
 
+/*!
+ * \brief Retrieve the sample rate of a format according to RTP specifications
+ * \since 16.7.0
+ * \since 17.1.0
+ *
+ * \param format The media format
+ *
+ * \retval The sample rate
+ */
+int ast_rtp_get_rate(const struct ast_format *format);
+
 /*!
  * \since 12
  * \brief \ref stasis topic for RTP and RTCP related messages
@@ -2708,6 +2820,84 @@ struct ast_json *ast_rtp_instance_get_stats_all_json(struct ast_rtp_instance *in
  */
 struct stasis_topic *ast_rtp_topic(void);
 
+/* RTP debug logging category name */
+#define AST_LOG_CATEGORY_RTP "rtp"
+/* RTP packet debug logging category name */
+#define AST_LOG_CATEGORY_RTP_PACKET "rtp_packet"
+/* RTCP debug logging category name */
+#define AST_LOG_CATEGORY_RTCP "rtcp"
+/* RTCP packet debug logging category name */
+#define AST_LOG_CATEGORY_RTCP_PACKET "rtcp_packet"
+/* DTLS debug logging category name */
+#define AST_LOG_CATEGORY_DTLS "dtls"
+/* DTLS packet debug logging category name */
+#define AST_LOG_CATEGORY_DTLS_PACKET "dtls_packet"
+/* ICE debug logging category name */
+#define AST_LOG_CATEGORY_ICE "ice"
+
+uintmax_t ast_debug_category_rtp_id(void);
+uintmax_t ast_debug_category_rtp_packet_id(void);
+uintmax_t ast_debug_category_rtcp_id(void);
+uintmax_t ast_debug_category_rtcp_packet_id(void);
+uintmax_t ast_debug_category_dtls_id(void);
+uintmax_t ast_debug_category_dtls_packet_id(void);
+uintmax_t ast_debug_category_ice_id(void);
+
+#define AST_DEBUG_CATEGORY_RTP ast_debug_category_rtp_id() /* RTP debug logging category id */
+#define AST_DEBUG_CATEGORY_RTP_PACKET ast_debug_category_rtp_packet_id() /* RTP packet debug logging category id */
+#define AST_DEBUG_CATEGORY_RTCP ast_debug_category_rtcp_id() /* RTCP debug logging category id */
+#define AST_DEBUG_CATEGORY_RTCP_PACKET ast_debug_category_rtcp_packet_id() /* RTCP packet debug logging category id */
+#define AST_DEBUG_CATEGORY_DTLS ast_debug_category_dtls_id() /* DTLS debug logging category id */
+#define AST_DEBUG_CATEGORY_DTLS_PACKET ast_debug_category_dtls_packet_id() /* DTLS packet debug logging category id */
+#define AST_DEBUG_CATEGORY_ICE ast_debug_category_ice_id() /* ICE debug logging category id */
+
+/*!
+ * \brief Log debug level RTP information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_rtp(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_RTP,  __VA_ARGS__)
+
+/* Allow logging of RTP packets? */
+#define ast_debug_rtp_packet_is_allowed \
+	ast_debug_category_is_allowed(AST_LOG_CATEGORY_ENABLED, AST_DEBUG_CATEGORY_RTP_PACKET)
+
+/*!
+ * \brief Log debug level RTCP information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_rtcp(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_RTCP, __VA_ARGS__)
+
+/* Allow logging of RTCP packets? */
+#define ast_debug_rtcp_packet_is_allowed \
+	ast_debug_category_is_allowed(AST_LOG_CATEGORY_ENABLED, AST_DEBUG_CATEGORY_RTCP_PACKET)
+
+/*!
+ * \brief Log debug level DTLS information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_dtls(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_DTLS, __VA_ARGS__)
+
+/* Allow logging of DTLS packets? */
+#define ast_debug_dtls_packet_is_allowed \
+	ast_debug_category_is_allowed(AST_LOG_CATEGORY_ENABLED, AST_DEBUG_CATEGORY_DTLS_PACKET)
+/*!
+ * \brief Log debug level ICE information
+ *
+ * \param sublevel Debug output sublevel (>= 0)
+ * \param ... String format and any associated arguments
+ */
+#define ast_debug_ice(sublevel, ...) \
+	ast_debug_category(sublevel, AST_DEBUG_CATEGORY_ICE, __VA_ARGS__)
+
 /* @} */
 
 #if defined(__cplusplus) || defined(c_plusplus)
